
# THIS FILE IS GENERATED FROM wofrysrw SETUP.PY
short_version = '1.1.0'
version = '1.1.0'
full_version = '1.1.0.dev0+a10ef63'
git_revision = 'a10ef631330ef36812bd003e026e960cfc367202'
release = False

if not release:
    version = full_version
    short_version += ".dev"
